import urllib3
from typing import List
from rich.live import Live
from rich.table import Table
import typer
import time
import re
import requests


urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)


def check_site(url_list: List[str]):
    """main function to check the status of websites."""
    # configuring table
    table = Table()
    table.add_column("WEBSITE", justify="center")
    table.add_column("STATUS", justify="center")
    table.add_column("Last Request", justify="center")

    # validating URLS
    url_list = [validate_url(urls) for urls in url_list]

    for urls in url_list:
        try:
            requests.get(url=urls, timeout=2.5, verify=False)
            table.add_row(urls, "[green]ONLINE", f"{time.strftime('%H:%M:%S', time.localtime())}")
        except requests.exceptions.ConnectionError: # if the site is not online, the program will raise an exception.
            table.add_row(urls, "[red]OFFLINE", f"{time.strftime('%H:%M:%S', time.localtime())}")
        
    return table


def validate_url(url: str):
    """adds "http" scheme before url if it doesn't exist."""
    pattern = r"(?i)\b((?:https?://|www\d{0,3}[.]|[a-z0-9.\-]+[.][a-z]{2,4}/)(?:[^\s()<>]+|\(([^\s()<>]+|(\([^\s()<>]+\)))*\))+(?:\(([^\s()<>]+|(\([^\s()<>]+\)))*\)|[^\s`!()\[\]{};:'\".,<>?«»“”‘’]))"

    if not re.match(pattern, url):
        return "http://" + url
    return url


def process_url(url: List[str] = typer.Option(..., "-u", "--url", help="URL of the site that needs to be checked."), cooldown: int = typer.Option(3, "-c", "--cooldown", help="Cooldown for the next request.")):
    """Zeus the Investigator: Checks if a site is online at the moment."""
    with Live(check_site(url), refresh_per_second=4) as live:
        live.console.print("\nInvestigator initiated!(Press [bold blue]CTRL+C[/] to quit)\n")
        while True:
            time.sleep(cooldown)
            live.update(check_site(url))


def main():
    typer.run(process_url)


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['zeustheinvestigator']

package_data = \
{'': ['*']}

install_requires = \
['requests>=2.27.1,<3.0.0',
 'rich>=12.4.3,<13.0.0',
 'typer>=0.4.1,<0.5.0',
 'urllib3>=1.26.9,<2.0.0']

entry_points = \
{'console_scripts': ['zeus = ZeusTheInvestigator.checker:main']}

setup_kwargs = {
    'name': 'zeustheinvestigator',
    'version': '0.1.4',
    'description': 'A pure python program that checks if a site is online at the moment',
    'long_description': '# Zeus the Investigator\nA pure python program that checks if a site is online/offline at the moment.\n\n#### **USAGE**\n```powershell\n$ zeus -u <first_url> -u <second_url> -c <cooldown(in secs. default=3)>\n```\n\n# Installation\n## Manual Installation\n**Note:** *You will need poetry for manual installation*\n```bash\n$ pip install poetry\n```\n\n1. Download or clone the repository.\n```git\n$ git clone https://github.com/777advait/Zeus-The-Investigator\n```\n\n2. Install the project by running the following command in the root of the directory.\n``` bash\n$ poetry install\n```\n\n\n## PyPi Installation\n```bash\n$ pip install ZeusTheInvestigator\n```',
    'author': '777advait',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/777advait/Zeus-The-Investigator',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
